# MUMPS Installation
MESTI.jl uses the parallel version of MUMPS for the augmented partial factorization (APF) method, and optionally for the factorize-and-solve method. Here are the steps to install MUMPS.

## Download MUMPS
Go to the [MUMPS website](https://graal.ens-lyon.fr/MUMPS/index.php?page=dwnld) and fill out the download request form. The MUMPS maintainers will email you the download link.

## Prerequisites
To compile the parallel version of MUMPS, you need compilation tools like <code>make</code>, <code>ar</code>, and <code>ranlib</code>, C and Fortran compilers, BLAS library, LAPACK library, ScaLAPACK library, and MPI library. Instructions specific to the operating system are provided below:
1. [Linux](./linux)
2. [macOS](./macOS)
3. [Windows](./windows)

If you are interested in 3D systems or memory usage in 2D systems is important for you, we highly recommend you install the [METIS](https://github.com/scivision/METIS/tree/743ae96033f31907d89c80e3470c0325e9a97f7b) (version 5.1.0) program for graph partitioning (not to be confused with MESTI). You can find the installation instructions in the operating system folders. After installing METIS, if you set <code>opts.use_METIS = true</code> in <code>mesti()</code> or <code>mesti2s()</code>, MUMPS will use METIS for matrix ordering. From our experience, in 2D, AMD is usually faster when using the APF method, but METIS can sometimes reduce memory usage. In 3D, METIS is strongly recommended, which is much faster than AMD. By default, 2D systems use AMD, while 3D systems use METIS (if it is available).

## Compile MUMPS
Suppose you downloaded the 5.6.2 version of MUMPS to your ~/Downloads/ folder. Then, go to the folder where you want to compile MUMPS, and enter
```
tar zxvf ~/Downloads/MUMPS_5.6.2.tar.gz
cd MUMPS_5.6.2
```
in terminal.

Read the file <code>INSTALL</code>, copy the closest <code>Makefile.inc</code> file in the <code>Make.inc</code> folder, and modify it to fit your environment and machine. Most importantly, in <code>Makefile.inc</code> you need to specify:
 - <code>CC</code>: the C compiler to use
 - <code>FC</code>: the Fortran compiler to use
 - <code>FL</code>: the Fortran linker to use
 - <code>LAPACK</code>: how the Fortran compiler can link to the LAPACK library
 - <code>SCALAP</code>: how the Fortran compiler can link to the ScaLAPACK library
 - <code>LIBBLAS</code>: how the Fortran compiler can link to the BLAS library
 - <code>RPATH_OPT</code>: the path the shared libraries will be built up, such as <code>/home/hclin/MUMPS_5.6.2/lib/</code>

Note that from our experience, <code>RPATH_OPT</code> must be specified to successfully install the parallel version of MUMPS on Linux and Windows.

If you installed METIS, you also need to specify
 - <code>LMETISDIR</code>: path to the folder where the METIS library is
 - <code>IMETIS</code>: path to the folder where the METIS headers are

and add <code>-Dmetis</code> to <code>ORDERINGSF</code>.

Examples of <code>Makefile.inc</code> are provided below:
1. [Linux](./linux/Makefile.inc)
2. [macOS](./macOS/Makefile.inc)
3. [Windows](./windows/Makefile.inc)

To download, click the link above, click on the "Raw" button, and right-click to save the file. If the browser adds a .txt file extension, rename it to remove the txt extension.

After done with <code>Makefile.inc</code>, enter
```
make allshared
```
in the terminal, which will compile the parallel version of MUMPS with single and double precision for real and complex variables (*i.e.*, <code>smumps</code>, <code>dmumps</code>, <code>cmumps</code>, <code>zmumps</code>).

If there is no error, check if the following files have been generated in the <code>lib</code> folder: <code>libsmumps.so</code>, <code>libdmumps.so</code>, <code>libcmumps.so</code>, and <code>libzmumps.so</code>.

Warning messages from the Fortran compiler are normal and can be ignored.

If there is an error, read the message and try to figure out where it comes from and/or look it up and address it. Before recompiling with <code>make allshared</code>, be sure to type <code>make clean</code> first to remove files generated by the previous compile attempt.

## Install Julia, Julia interface for MUMPS, and necessary packages in Julia

Follow the standard process to download and install Julia. We can download Julia [here](https://julialang.org/downloads/). Suppose we installed the 1.9.3 version of Julia. After Julia is installed, we can add the path of your Julia to <code>PATH</code> by 

    export PATH=".../julia-1.9.3/bin/"

where  <code>... </code> is the path to your Julia.

Then run the provided <code>[install_packages.jl](./install_packages.jl)</code>

    julia install_packages.jl

It helps us to add the Julia interface for MUMPS, [MUMPS3](https://github.com/wrs28/MUMPS3.jl/tree/5.3.3-update), and other necessary packages for MESTI.jl.


## Test MUMPS

Now, open Julia and the following test scripts:

- <code>[basic_solve.jl](basic_solve.jl)</code>
- <code>[schur_complement.jl](schur_complement.jl)</code>

Please modify the line 6 and set the environment variable <code>ENV["MUMPS_PREFIX"]</code> to the path of MUMPS library on your machine. Then, we can run these two scripts.

If any of them does not run successfully, look back at the compilation of MUMPS or the Julia interface to see if there were serious warning messages.

If they all pass, congratulations! You are done.

If you would like to use METIS, but your cluster cannot find METIS libraries by itself when you run Julia interface for MUMPS. You can append the METIS libraries to your `LD_LIBRARYP_PATH`

```
export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$LMETISDIR
```

`LMETISDIR` is the path to the folder where the METIS library is.
